{% load i18n %}
<p>
    {% comment %}Translators: The text on the button the user clicks to get to the next page{% endcomment %}
        <input class="btn btn-primary btn-large btn-primary next-button" type="submit" value="{% blocktrans %}Next{% endblocktrans %}" />
</p>
