{% load staticfiles %}
<script src="{% static 'otree/js/reconnecting-websocket.js' %}" type="text/javascript"></script>

<!-- compiled and minified JavaScript -->
<script src="{% static 'bootstrap/js/bootstrap.min.js' %}"></script>

<!-- HTML5 shim and Respond.js IE8 support of HTML5 elements and media queries -->
<!--[if lt IE 9]>
  <script src="../../assets/js/html5shiv.js"></script>
  <script src="../../assets/js/respond.min.js"></script>
<![endif]-->

<script src="{% static 'otree/js/init_widgets.js' %}"></script>
