{% extends "global/Base.html" %}
{% load staticfiles otree_tags %}

{% block title %}
    Page title
{% endblock %}

{% block content %}

<!--
Documentation on how to write a django template here:
https://docs.djangoproject.com/en/1.8/ref/templates/language/

Bootstrap comes pre-loaded, so you can use any Bootstrap elements:
http://getbootstrap.com/components/

-->

    {% next_button %}

{% endblock %}
