{% load staticfiles %}
{% load i18n %}

<div id="time-limit" class="alert alert-warning" style="display:none;">
    <div id="clock">
        <p>{% blocktrans %}Time left to complete this page:{% endblocktrans %}
            <span style="font-weight: bold">
                    <span class="glyphicon glyphicon-time"></span>
                    <span id="minutes"></span>:<span id="seconds"></span>
            </span>
        </p>
    </div>
</div>
