{% extends "otree/BaseParticipant.html" %}
{% load i18n %}

{% block title %}
    Finished
{% endblock %}

{% block content %}

    <p>
        {% blocktrans %}No more pages left to show.{% endblocktrans %}
    </p>

{% endblock %}
